package com.yobotics.exampleSimulations.fallingBrick;

import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

import com.yobotics.simulationconstructionset.GroundProfile;

public class WavyGroundProfile implements GroundProfile
{
   private double xMin = -2.0, xMax = 2.0, yMin = -2.0, yMax = 2.0;
   private double xTiles = 1.0, yTiles = 1.0;

   public WavyGroundProfile()
   {
   }

   public double heightAt(double x, double y, double z)
   {
      if ((x > xMin) && (x < xMax) && (y > yMin) && (y < yMax))
         return 1.0 * Math.exp(-Math.abs(2.0 * x)) * Math.exp(-Math.abs(2.0 * y)) * Math.sin(2.0 * Math.PI * 0.7 * x);
      else
         return 0.0;
   }

   public void surfaceNormalAt(double x, double y, double z, Vector3d normal)
   {
      normal.x = 0.0;
      normal.y = 0.0;
      normal.z = 1.0;
   }

   public void closestIntersectionTo(double x, double y, double z, Point3d point)
   {
      point.x = x;
      point.y = y;
      point.z = heightAt(x, y, z);
   }

   public void closestIntersectionAndNormalAt(double x, double y, double z, Point3d point, Vector3d normal)
   {
      closestIntersectionTo(x, y, z, point);
      surfaceNormalAt(x, y, z, normal);
   }


   public boolean isClose(double x, double y, double z)
   {
      return true;
   }

   public double getXMin()
   {
      return xMin;
   }

   public double getYMin()
   {
      return yMin;
   }

   public double getXMax()
   {
      return xMax;
   }

   public double getYMax()
   {
      return yMax;
   }

   public double getXTiles()
   {
      return xTiles;
   }

   public double getYTiles()
   {
      return yTiles;
   }

}
